# This master make file is based on the "old"
# systemvars.mk and externallibs.mk files from:
# 	apple-darwin13-llvm6.0
# 	linux_64-gcc4.4
# 	linux_64-gcc4.8

# get system type (Darwin, Linux)
SYSTYPE := $(shell uname -s)

#####################################################################
#
# Standard commands
#
#####################################################################
SHELL   ?= /bin/sh
RM      ?= /bin/rm
CP      ?= /bin/cp
MV      ?= /bin/mv
CHMOD   ?= /bin/chmod
MKDIR   ?= /bin/mkdir
INSTALL ?= install -p
TCLSH   ?= ${FSLDIR}/bin/fsltclsh

#####################################################################
#
# Architecture-specific compiler/linker flags
#
#####################################################################

# All platform specific compiler/linker options are added to
# the following variables. They are populated in the platform-
# specific branches below. Final values for CFLAGS, CXXFLAGS,
# and LDFLAGS (including include/library search paths) are
# constructed in common/vars.mk

# Compiler flags for C projects
ARCHCFLAGS =

# Compiler flags for C++ projects.
ARCHCXXFLAGS = -D_GLIBCXX_USE_CXX11_ABI=0

# Linker flags for all projects
ARCHLDFLAGS =

# Libraries available for linking by all projects.
ARCHLIBS =

# Compiler flags for CUDA projects.
ARCHNVCCFLAGS =

# Linker flags for CUDA projects.
ARCHNVCCLDFLAGS =

#####################################################################
#
# Darwin specific sys vars and ext libs
#
#####################################################################
ifeq ($(SYSTYPE), Darwin)

  # Commands
  CC  ?= cc
  CXX ?= c++

  # Compiler flags
  ARCHCXXFLAGS += -arch x86_64 -Wall -pedantic -fPIC
  ARCHCFLAGS   += $(ARCHCXXFLAGS) -std=c99
  ARCHLDFLAGS  += -Wl,-search_paths_first
  ARCHLIBS     += -llapack -lblas -lz -lm
endif

#####################################################################
#
# Linux specific sys vars and ext libs
#
#####################################################################
ifeq ($(SYSTYPE), Linux)

 # Commands
  CC  ?= gcc
  CXX ?= c++

  # Compiler flags
  ARCHCFLAGS   += -fexpensive-optimizations \
                  -Wall -pedantic -Wno-long-long -fPIC
  ARCHCXXFLAGS += $(ARCHCFLAGS)
  ARCHLDFLAGS  += -Wl,-rpath,'$$ORIGIN/../lib'
  ARCHLIBS     += -lopenblas -lz -lm
endif # if Linux

#####################################################################
#
# CUDA specific sys vars and ext libs
#
#####################################################################

# CUDA/nvcc compiler options
ifneq ($(shell which nvcc), )

  NVCC         ?= nvcc
  CUDAVER      ?= $(shell nvcc --version | grep -Po "release \K[0-9\.]+")
  GENCODEFLAGS ?= $(shell ${FSLCONFDIR}/common/supportedGencodes.sh ${CUDAVER})
  CUDADIR      ?= ${FSLDIR}

  # nvcc-specific tweaks:
  #  - Define ARMA_ALLOW_FAKE_GCC, otherwise byte-alignment issues
  #    may occur with armadillo datatypes (see compiler_setup.hpp
  #    from the armadillo project)
  #  - Do not use -pedantic, otherwise *every single line* of
  #    transpiled cuda code will trigger "warning: style of line
  #    directive is a GCC extension".
  #  - Specify the compiler to use with --compiler-bindir, as nvcc
  #    might otherwise naively call "gcc"
  ARCHNVCCFLAGS   = -I${CUDADIR}/include  \
                    -maxrregcount=64      \
                    ${GENCODEFLAGS}       \
                    -DARMA_ALLOW_FAKE_GCC \
                    --compiler-bindir     \
                    $(shell which $(CC))  \
                    --compiler-options    \
                    "$(filter-out -pedantic, ${ARCHCXXFLAGS})"
  ARCHNVCCLDFLAGS = -L${CUDADIR}/lib \
                    -L${CUDADIR}/lib/stubs
endif
