#########################################################
# Variables used for FSL project compilation/installation
#########################################################

# All projects must specify a name
PROJNAME =

# Project-specific compiler/linker flags - these
# can be overridden in the project Makefile.

USRLDFLAGS      = # Project-specific linker flags
USRINCFLAGS     = # Project-specific include directories
USRCFLAGS       = # Project-specific compiler flags for C projects
USRCXXFLAGS     = # Project-specific compiler flags for C++ projects
LIBS            = # Project-specific libraries to link against for C
                  # and C++ projects - these are incorporated into the
                  # final LDFLAGS, below.
USRNVCCFLAGS    = # Project-specific compiler flags for CUDA projects
USRNVCCLDFLAGS  = # Project-specific linker flags for CUDA projects
CUDALIBS        = # Project-specific libraries to link against for CUDA
                  # projects - these are incorporated into the final
                  # NVCCLDFLAGS, below.

# Project outputs - these variables control
# what gets installed from a project. They
# may be customised/overridden in project
# Makefiles.
HFILES       = *.h    # installed into $FSLDIR/include/$PROJNAME/
GLOBALHFILES =        # installed into $FSLDIR/include/ (should
                      # not be used in normal circumstances)
AFILES       = *.a    # installed into $FSLDIR/lib/
SOFILES      = *.so   # installed into $FSLDIR/lib/
XFILES       =        # installed into $FSLDIR/bin/
FXFILES      =        # installed into $FSLDIR/bin/, only for internal builds
SCRIPTS      =        # installed into $FSLDIR/bin/
FSCRIPTS     =        # installed into $FSLDIR/bin/, only for internal builds
RUNTCLS      =        # link to Runtcl created in $FSLDIR/bin/ which assumes
                      # that <file>.tcl has been installed into
                      # $FSLDIR/tcl/
PYFILES      = *.py   # installed into $FSLDIR/etc/fsl/python/$PROJNAME/
TCLFILES     = *.tcl  # installed into $FSLDIR/tcl/
DATAFILES    =        # installed into $FSLDIR/data/$PROJNAME/
TESTXILES    =        # not currently used

# Final install destinations
# for project outputs
DESTDIR      = ${FSLDEVDIR}
dest_INCDIR  = ${DESTDIR}/include
dest_LIBDIR  = ${DESTDIR}/lib
dest_BINDIR  = ${DESTDIR}/bin
dest_TCLDIR  = ${DESTDIR}/tcl
dest_PYDIR   = ${DESTDIR}/etc/fsl/python
dest_DOCDIR  = ${DESTDIR}/doc
dest_DATADIR = ${DESTDIR}/data

# Standard header/library locations for compiling/linking
INCDIR    = ${FSLDIR}/include
LIBDIR    = ${FSLDIR}/lib
DEVINCDIR = ${FSLDEVDIR}/include
DEVLIBDIR = ${FSLDEVDIR}/lib

# Final header include directories
INCFLAGS = ${USRINCFLAGS} -I. -I${DEVINCDIR} -I${INCDIR}

#############################
# Final compiler/linker flags
#############################

# Ignore flags provided by the environment
CFLAGS      =
CXXFLAGS    =
CPPFLAGS    =
NVCCFLAGS   =
LDFLAGS     =
NVCCLDFLAGS =

# All projects must use these flags for
# compilation/linking, using the form:
#
# $(CC)   $(CFLAGS)    <project flags/files> $(LDFLAGS)
# $(CXX)  $(CXXFLAGS)  <project flags/files> $(LDFLAGS)
# $(NVCC) $(NVCCFLAGS) <project flags/files> $(NVCCLDFLAGS)
#
# LDFLAGS *must* come at the end, to ensure proper linking.

LDFLAGS       += ${ARCHLDFLAGS} ${USRLDFLAGS} \
                 -L. -L${DEVLIBDIR} -L${LIBDIR} \
                  ${LIBS} ${ARCHLIBS}
NVCCLDFLAGS   += ${ARCHNVCCLDFLAGS} ${USRNVCCLDFLAGS} \
                 -L. -L${DEVLIBDIR} -L${LIBDIR} \
                 ${CUDALIBS}
CFLAGS        += ${ARCHCFLAGS}    ${USRCFLAGS}    ${INCFLAGS}
CXXFLAGS      += ${ARCHCXXFLAGS}  ${USRCXXFLAGS}  ${INCFLAGS}
NVCCFLAGS     += ${ARCHNVCCFLAGS} ${USRNVCCFLAGS} ${INCFLAGS}
